@@ -6,15 +6,22 @@
 #include "RcppParallel/TinyThread.h"
 
 // Use TBB only where it's known to compile and work correctly
+// (NOTE: Windows TBB is temporarily opt-in for packages for 
+// compatibility with CRAN packages not previously configured
+// to link to TBB in Makevars.win)
 #ifndef RCPP_PARALLEL_USE_TBB
 #if defined(__APPLE__) || defined(__gnu_linux__)
-  #define RCPP_PARALLEL_USE_TBB 1
-  #include "RcppParallel/TBB.h"
+#define RCPP_PARALLEL_USE_TBB 1
+#include "RcppParallel/TBB.h"
 #else
-  #define RCPP_PARALLEL_USE_TBB 0
+#define RCPP_PARALLEL_USE_TBB 0
 #endif
 #endif
 
+#if RCPP_PARALLEL_USE_TBB
+  #include "RcppParallel/TBB.h"
+#endif
+
 #include "RcppParallel/RVector.h"
 #include "RcppParallel/RMatrix.h"
 
